/* Cyclops
 * FER - SPVP
 * Fakultet elektrotehnike i racunarstva (http://www.fer.hr/)
 * Unska 3, 10000 Zagreb, Hrvatska
 * (c) 2001 FER, Zagreb.
 */

package hr.fer.zesoi.cyclops;

import java.util.*;


/**
 * The Queue class implements a generic buffer required for packet storage.
 * Implemented as stack (JDK 1.1.8. does not support linked lists).
 * @author Darko Vasi
 * @author Tomislav Petkovi
 * @author Zvonko Kostanjar
 */
public class Queue
{

    /**
     * Maximal number of packets that can pass through the line
     * in one simulation cycle.
     */
    protected int max_capacity;

    /**
     * Number of packets currently stored in a buffer.
     */
    protected int number_of_packets;

    /**
     * Packet storage.
     */
    private Stack storage_buffer = new Stack();

    /**
     * Default constructor. One packet buffer is created.
     */
    public Queue ()
    {
	max_capacity = 1;
	number_of_packets = 0;
    }
    /* Queue */

    /**
     * Normal constructor. Buffer of specified capacity is created.
     */
    public Queue (int capacity)
    {
	max_capacity = (capacity < 0)? 0: capacity;
	number_of_packets = 0;
    }
    /* Queue */

    /**
     * Sets buffer capacity to specified capacity. If buffer contains
     * more elements then specified capacity, some elements are lost.
     * @param capacity Buffer capacity.
     */
    public void setCapacity (int capacity)
    {
	while (capacity < number_of_packets)
	    {
		this.get();
	    }
	this.max_capacity = capacity;
    }
    /* setCapacity */

    /**
     * Test for empty buffer. Returns true if empty.
     * @return Buffer status.
     */
    public boolean isEmpty ()
    {
	return storage_buffer.isEmpty();
    }
    /* isEmpty */

    /**
     * Puts one packet in the buffer. Returns true
     * if succesful, and false if buffer is full.
     * @param packet New element.
     * @return Returns false for full buffer.
     */
    public boolean put (Object packet)
    {
	if (number_of_packets <= max_capacity)
	    {
		storage_buffer.push(packet);
		number_of_packets = number_of_packets + 1;
		return true;
	    }
	else
	    {
		return false;
	    }
    }
    /* put */

    /**
     * Gets next element (packet) and removes it from the
     * buffer. Generates exception if buffer is empty.
     * @return Returns buffer element (packet).
     */
    public Object get ()
    {
	Object packet;

	if (number_of_packets > 0)
	    {
		number_of_packets = number_of_packets - 1;
		packet = storage_buffer.pop();
		return packet;
	    }
	else
	    {
		throw new NoSuchElementException("No more packages.");
	    }
    }
    /* get */

    /**
     * Peeks at the first element.
     *
     * @return Returns first element.
     */
    public Object peek ()
    {
	return storage_buffer.peek();
    }
    /* peek */

}
/* Queue */
